﻿using System;
using System.Runtime.InteropServices.WindowsRuntime;
using System.Threading.Tasks;
using Windows.ApplicationModel;
using Windows.ApplicationModel.Appointments;
using Windows.ApplicationModel.Contacts;
using Windows.ApplicationModel.Search;
using Windows.Data.Pdf;
using Windows.Foundation;
using Windows.Graphics.Display;
using Windows.Graphics.Imaging;
using Windows.Media.SpeechSynthesis;
using Windows.Storage;
using Windows.Storage.Streams;
using Windows.UI.ApplicationSettings;
using Windows.UI.Popups;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Media;
// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234238
using Windows.UI.Xaml.Media.Imaging;

namespace WhatsNewWin81
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainPage : Page
    {
		private string _id;

        public MainPage()
        {
            this.InitializeComponent();
        }

		private async void AddAppointment_Click(object sender, RoutedEventArgs e)
		{
            Appointment appointment = new Appointment
            {
	            Subject = "My appointment!",
	            StartTime = DateTimeOffset.Now,
	            Duration = TimeSpan.FromHours(2)
            };

			Rect rect = GetElementRect(sender as FrameworkElement);

            _id = await AppointmentManager.ShowAddAppointmentAsync(appointment, rect, Placement.Default);
			await new MessageDialog("Appointment Id: " + _id).ShowAsync();
		}

		private async void RemoveApppointment_Click(object sender, RoutedEventArgs e)
		{
			Rect rect = GetElementRect(sender as FrameworkElement);
			bool removed = await AppointmentManager.ShowRemoveAppointmentAsync(_id, rect, Placement.Default);
			await new MessageDialog("Removed: " + removed).ShowAsync();
		}

		private void ShowContactCard_Click(object sender, RoutedEventArgs e)
		{
            Contact contact = new Contact();
            ContactEmail email = new ContactEmail {Address = "brian@brianpeek.com"};
			contact.Emails.Add(email);

            Rect rect = GetElementRect(sender as FrameworkElement);

            ContactManager.ShowContactCard(contact, rect, Placement.Default);
		}

	    private async void ShowDelayedContactCard_Click(object sender, RoutedEventArgs e)
	    {
            Contact contact = new Contact {FirstName = "Emmet", LastName = "Brown"};

		    ContactEmail email = new ContactEmail {Address = "doc@hillvalley.com"};
		    contact.Emails.Add(email);

            Rect rect = GetElementRect(sender as FrameworkElement);
            using (ContactCardDelayedDataLoader dataLoader = ContactManager.ShowDelayLoadedContactCard(contact, rect, Placement.Below))
            {
                Contact fullContact = await DownloadContactDataAsync(contact);
                if (fullContact != null)
                    dataLoader.SetData(fullContact);
            }
		}

        private async Task<Contact> DownloadContactDataAsync(Contact contact)
        {
            await Task.Delay(2000);

            return await Task.Run(() =>
            {
                ContactPhone homePhone = new ContactPhone {Number = "(916) 555-4385", Kind = ContactPhoneKind.Home};
	            contact.Phones.Add(homePhone);

                ContactAddress address = new ContactAddress
                {
	                StreetAddress = "1640 Riverside Dr",
	                Locality = "Hill Valley",
	                Region = "CA",
	                Country = "USA",
	                PostalCode = "95420",
	                Kind = ContactAddressKind.Home
                };
	            contact.Addresses.Add(address);

				contact.Thumbnail = RandomAccessStreamReference.CreateFromUri(new Uri("https://dl.dropboxusercontent.com/u/12493073/docbrown.png"));

                return contact;
            });
        }

		private void SearchBox_SuggestionsRequested(SearchBox sender, SearchBoxSuggestionsRequestedEventArgs args)
		{
            if (!string.IsNullOrEmpty(args.QueryText))
            {
                SearchSuggestionCollection suggestionCollection = args.Request.SearchSuggestionCollection;

                foreach (string suggestion in suggestionList)
                {
                    if (suggestion.StartsWith(args.QueryText, StringComparison.CurrentCultureIgnoreCase))
                        suggestionCollection.AppendQuerySuggestion(suggestion);
                }
            }
		}

		private async void SearchBox_QuerySubmitted(SearchBox sender, SearchBoxQuerySubmittedEventArgs args)
		{
			await new MessageDialog("Query: " + args.QueryText).ShowAsync();
		}

		private async void Speak_Click(object sender, RoutedEventArgs e)
		{
			SpeechSynthesizer synthesizer = new SpeechSynthesizer();

			//synthesizer.Voice = SpeechSynthesizer.AllVoices[1];
            SpeechSynthesisStream synthesisStream = await synthesizer.SynthesizeTextToStreamAsync("Great Scott!");

            MediaPlayer.AutoPlay = true;
            MediaPlayer.SetSource(synthesisStream, synthesisStream.ContentType);
            MediaPlayer.Play();
		}

		protected override void OnNavigatedTo(Windows.UI.Xaml.Navigation.NavigationEventArgs e)
		{
			SettingsPane.GetForCurrentView().CommandsRequested += MainPage_CommandsRequested;
		}

        void MainPage_CommandsRequested(SettingsPane settingsPane, SettingsPaneCommandsRequestedEventArgs e)
        {
            SettingsCommand defaultsCommand = new SettingsCommand("defaults", "Defaults", (handler) =>
            {
                SettingsFlyout1 sf = new SettingsFlyout1();
                sf.Show();
            });
            e.Request.ApplicationCommands.Add(defaultsCommand);
        }

		private async void SaveScreenshot_Click(object sender, RoutedEventArgs e)
		{
			RenderTargetBitmap renderTargetBitmap = new RenderTargetBitmap();
			await renderTargetBitmap.RenderAsync(RootPanel);
			var pixelBuffer = await renderTargetBitmap.GetPixelsAsync();

			StorageFile saveFile = await KnownFolders.PicturesLibrary.CreateFileAsync("screenshot.png");
			using (var fileStream = await saveFile.OpenAsync(FileAccessMode.ReadWrite))
			{
				var encoder = await BitmapEncoder.CreateAsync(BitmapEncoder.PngEncoderId, fileStream);
				encoder.SetPixelData(
					BitmapPixelFormat.Bgra8,
					BitmapAlphaMode.Ignore,
					(uint)renderTargetBitmap.PixelWidth,
					(uint)renderTargetBitmap.PixelHeight,
					DisplayInformation.GetForCurrentView().LogicalDpi,
					DisplayInformation.GetForCurrentView().LogicalDpi,
					pixelBuffer.ToArray());
				await encoder.FlushAsync();
			}

			await new MessageDialog("Done!").ShowAsync();
		}

		private async void Hyperlink_Click(object sender, RoutedEventArgs e)
		{
			await new MessageDialog("You clicked the hyperlink!").ShowAsync();
		}

		private PdfDocument _doc;
		private uint _page;

		private async void LoadPdf_Click(object sender, RoutedEventArgs e)
		{
			StorageFile file = await Package.Current.InstalledLocation.GetFileAsync("bttf4th.pdf");
			_doc = await PdfDocument.LoadFromFileAsync(file);
			RenderPage(_page++);
		}

	    private void Previous_Click(object sender, RoutedEventArgs e)
	    {
			if(_page == 0)
				return;

			RenderPage(--_page);
	    }

		private void Next_Click(object sender, RoutedEventArgs e)
		{
			if(_page == _doc.PageCount)
				return;

			RenderPage(_page++);
		}

		private async void RenderPage(uint pageNum)
		{
			PdfPage page = _doc.GetPage(pageNum);
			InMemoryRandomAccessStream stream = new InMemoryRandomAccessStream();
			await page.RenderToStreamAsync(stream);
			BitmapImage bi = new BitmapImage();
			bi.SetSource(stream);
			Image.Source = bi;
		}

        private Rect GetElementRect(FrameworkElement element)
        {
            GeneralTransform buttonTransform = element.TransformToVisual(null);
            Point point = buttonTransform.TransformPoint(new Point());
            return new Rect(point, new Size(element.ActualWidth, element.ActualHeight));
        }

        private static readonly string[] suggestionList =
        {
            "Shanghai", "Istanbul", "Karachi", "Delhi", "Mumbai", "Moscow", "São Paulo", "Seoul", "Beijing", "Jakarta",
            "Tokyo", "Mexico City", "Kinshasa", "New York City", "Lagos", "London", "Lima", "Bogota", "Tehran", "Ho Chi Minh City",
            "Hong Kong", "Bangkok", "Dhaka", "Cairo", "Hanoi", "Rio de Janeiro", "Lahore", "Chonquing", "Bengaluru", "Tianjin",
            "Baghdad", "Riyadh", "Singapore", "Santiago", "Saint Petersburg", "Surat", "Chennai", "Kolkata", "Yangon", "Guangzhou",
            "Alexandria", "Shenyang", "Hyderabad", "Ahmedabad", "Ankara", "Johannesburg", "Wuhan", "Los Angeles", "Yokohama",
            "Abidjan", "Busan", "Cape Town", "Durban", "Pune", "Jeddah", "Berlin", "Pyongyang", "Kanpur", "Madrid", "Jaipur",
            "Nairobi", "Chicago", "Houston", "Philadelphia", "Phoenix", "San Antonio", "San Diego", "Dallas", "San Jose",
            "Jacksonville", "Indianapolis", "San Francisco", "Austin", "Columbus", "Fort Worth", "Charlotte", "Detroit",
            "El Paso", "Memphis", "Baltimore", "Boston", "Seattle Washington", "Nashville", "Denver", "Louisville", "Milwaukee",
            "Portland", "Las Vegas", "Oklahoma City", "Albuquerque", "Tucson", "Fresno", "Sacramento", "Long Beach", "Kansas City",
            "Mesa", "Virginia Beach", "Atlanta", "Colorado Springs", "Omaha", "Raleigh", "Miami", "Cleveland", "Tulsa", "Oakland",
            "Minneapolis", "Wichita", "Arlington", "Bakersfield", "New Orleans", "Honolulu", "Anaheim", "Tampa", "Aurora",
            "Santa Ana", "St. Louis", "Pittsburgh", "Corpus Christi", "Riverside", "Cincinnati", "Lexington", "Anchorage",
            "Stockton", "Toledo", "St. Paul", "Newark", "Greensboro", "Buffalo", "Plano", "Lincoln", "Henderson", "Fort Wayne",
            "Jersey City", "St. Petersburg", "Chula Vista", "Norfolk", "Orlando", "Chandler", "Laredo", "Madison", "Winston-Salem",
            "Lubbock", "Baton Rouge", "Durham", "Garland", "Glendale", "Reno", "Hialeah", "Chesapeake", "Scottsdale",
            "North Las Vegas", "Irving", "Fremont", "Irvine", "Birmingham", "Rochester", "San Bernardino", "Spokane",
            "Toronto", "Montreal", "Vancouver", "Ottawa-Gatineau", "Calgary", "Edmonton", "Quebec City", "Winnipeg", "Hamilton"
        };
    }
}
